@@ -1,6 +1,5 @@
-// Copyright (c) 2021 Tailscale Inc & AUTHORS All rights reserved.
-// Use of this source code is governed by a BSD-style
-// license that can be found in the LICENSE file.
+// Copyright (c) Tailscale Inc & AUTHORS
+// SPDX-License-Identifier: BSD-3-Clause
 
 // Package multierr provides a simple multiple-error type.
 // It was inspired by github.com/go-multierror/multierror.
@@ -34,15 +33,12 @@ func (e Error) Errors() []error {
 	return slices.Clone(e.errs)
 }
 
-// TODO(https://go.dev/cl/53435): Implement Unwrap when Go 1.20 is released.
-/*
-// Unwrap returns the underlying errors as is.
+// Unwrap returns the underlying errors as-is.
 func (e Error) Unwrap() []error {
 	// Do not clone since Unwrap requires callers to not mutate the slice.
 	// See the documentation in the Go "errors" package.
 	return e.errs
 }
-*/
 
 // New returns an error composed from errs.
 // Some errors in errs get special treatment:
@@ -136,13 +132,6 @@ func Range(err error, fn func(error) bool) bool {
 				return false
 			}
 		}
-	// TODO(https://go.dev/cl/53435): Delete this when Error implements Unwrap.
-	case Error:
-		for _, err := range err.errs {
-			if !Range(err, fn) {
-				return false
-			}
-		}
 	}
 	return true
 }
